#include "CharSetMap.h"

// constructor
CharSetMap::CharSetMap ()
{
	numOfChars = 0;
	// initialize all code page pointers to NULL 
	for (int i = 0; i < __ULE_CHARSET_PAGE_SIZE; i++)
		CodePages[i] = NULL;
}

// copy constructor
CharSetMap::CharSetMap (const CharSetMap & r)
{
	numOfChars = r.numOfChars;
	
	for (int i = 0; i < __ULE_CHARSET_PAGE_SIZE; i++)
	{
		// ith code page exists in the right side obj
		if (r.CodePages[i] != NULL)
		{
			// allocate memeory for ith code page in left side obj
			CodePages[i] = new Word [__ULE_CHARSET_PAGE_SIZE];
			if (CodePages[i] == NULL)   // if could not allocate memory
				return;

			// copy entire array from right side object to left side obj
			memcpy ((void*)CodePages[i], (void*)r.CodePages[i], 
					__ULE_CHARSET_PAGE_SIZE*sizeof(Word));
		}
		else
			CodePages[i] = NULL;
	}
}

// destructor
CharSetMap::~CharSetMap ()
{
	Clear();
}

int CharSetMap::GetTotalNumOfChars ()
{
	return numOfChars;
}

// finds index of given char in O(1); returns UNDEF_CHAR if not persent in defined char set
unsigned short CharSetMap::GetIndex (wchar_t wc)
{
	Byte chHighByte;
	Byte chLowByte;

	//chLowByte = unsigned char(wc);
	//chHighByte = unsigned char(wc >> 8);	
	chLowByte = char(wc);
	chHighByte = char(wc >> 8);

	if (CodePages[chHighByte] != NULL)
	{
		// return index of given char (of invalid code UNDEF_CHAR if it is not present)
		return CodePages[chHighByte][chLowByte];
	}

	return UNDEF_CHAR;	// code page not found for given char
}

int CharSetMap::Insert(wchar_t wc)
{
	Byte chLowByte = Byte (wc);
	Byte chHighByte = Byte (wc >> 8);

	// check code page for this char is not already created
	if (CodePages[chHighByte] == NULL)
	{
		CodePages[chHighByte] = new Word [__ULE_CHARSET_PAGE_SIZE];
		if (CodePages[chHighByte] == NULL)   // if could not allocate memory
			return false;

		// initialize all its elements with UNDEF_CHAR to mark empty
		memset ((void*)(CodePages[chHighByte]), (Byte)UNDEF_CHAR, 
				__ULE_CHARSET_PAGE_SIZE*sizeof(Word));
	}

	// insert sequential index number to cur char's loc in its corresponding code page
	CodePages[chHighByte][chLowByte] = numOfChars++;
	return 1;
}

void CharSetMap::Clear()
{
	// de-allocate space for code pages
	for (int i = 0; i < __ULE_CHARSET_PAGE_SIZE; i++)
		if (CodePages[i] != NULL)
		{
			delete [] CodePages[i];
			CodePages[i] = NULL;
		}

	numOfChars = 0;

	return;
}
